#include "mex.hpp"
#include "mexAdapter.hpp"
#include "LabJackM.h"

using namespace matlab::data;
using matlab::mex::ArgumentList;

class MexFunction : public matlab::mex::Function {
public:
    void operator()(ArgumentList outputs, ArgumentList inputs) {
        int32_t handle, error;
        ArrayFactory factory;

        checkArguments(outputs, inputs);
        TypedArray<int32_t> handleArr = inputs[0];
        handle = handleArr[0];
        error = LJM_Close(handle);
        outputs[0] = factory.createScalar<int32_t>(error);
    }
private:
    void checkArguments(ArgumentList outputs, ArgumentList inputs) {
        std::shared_ptr<matlab::engine::MATLABEngine> matlabPtr = getEngine();
        ArrayFactory factory;
        if (inputs.size() != 1)
        {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("LJM_Close expects one input argument") }));
        }
        if (inputs[0].getType() != ArrayType::INT32 ||
            inputs[0].getType() == ArrayType::COMPLEX_INT32)
        {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("LJM_Close: Input 0 (handle) must be a noncomplex int32") }));
        }

        if (outputs.size() > 1) {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("LJM_Close: Only one output (error) is returned") }));
        }
    }
    // void ReturnError(LJM_ERROR_RETURN err) {
    //  std::shared_ptr<matlab::engine::MATLABEngine> matlabPtr = getEngine();
    //  ArrayFactory factory;
    //  std::string message
    //  matlabPtr->feval(u"error", 0,
    //             std::vector<Array>({ factory.createScalar("LJM Error %d: %s", err, message) }));
    // }
};