#include "mex.hpp"
#include "mexAdapter.hpp"
#include "LabJackM.h"

using namespace matlab::data;
using matlab::mex::ArgumentList;

class MexFunction : public matlab::mex::Function {
public:
    void operator()(ArgumentList outputs, ArgumentList inputs) {
	    int32_t ErrorCode;
        char * ErrorString;
		ArrayFactory factory;
        ErrorString = (char *)malloc(LJM_MAX_NAME_SIZE*sizeof(char));

        checkArguments(outputs, inputs);
        TypedArray<int32_t> errCodeArr = inputs[0];
        ErrorCode = errCodeArr[0];
        LJM_ErrorToString(ErrorCode, ErrorString);
        outputs[0] =  factory.createScalar(ErrorString);
        free(ErrorString);
    }
private:
    void checkArguments(ArgumentList outputs, ArgumentList inputs) {
        std::shared_ptr<matlab::engine::MATLABEngine> matlabPtr = getEngine();
        ArrayFactory factory;
        if (inputs.size() != 1)
        {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("LJM_ErrorToString expects one input argument") }));
        }
        if (inputs[0].getType() != ArrayType::INT32 ||
            inputs[0].getType() == ArrayType::COMPLEX_INT32)
        {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("LJM_ErrorToString: Input 0 (ErrorCode) must be a noncomplex int32") }));
        }

        if (outputs.size() > 1) {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("LJM_ErrorToString: Only one output (ErrorString) is returned") }));
        }
    }
    // void ReturnError(LJM_ERROR_RETURN err) {
    // 	std::shared_ptr<matlab::engine::MATLABEngine> matlabPtr = getEngine();
    // 	ArrayFactory factory;
    // 	std::string message
    // 	matlabPtr->feval(u"error", 0,
    //             std::vector<Array>({ factory.createScalar("LJM Error %d: %s", err, message) }));
    // }
};