#include "mex.hpp"
#include "mexAdapter.hpp"
#include "LabJackM.h"

using namespace matlab::data;
using matlab::mex::ArgumentList;

class MexFunction : public matlab::mex::Function {
public:
    void operator()(ArgumentList outputs, ArgumentList inputs) {
    	std::string DeviceType;
	    std::string ConnectionType;
	    std::string Identifier;
	    int32_t handle, error;
		ArrayFactory factory;

        checkArguments(outputs, inputs);
        TypedArray<MATLABString> devTypeArr = inputs[0];
        DeviceType = std::string(devTypeArr[0]);
        TypedArray<MATLABString> connTypeArr = inputs[1];
        ConnectionType = std::string(connTypeArr[0]);
        TypedArray<MATLABString> idArr = inputs[2];
        Identifier = std::string(idArr[0]);
        error = LJM_OpenS(DeviceType.c_str(), ConnectionType.c_str(), Identifier.c_str(), &handle);
        outputs[0] = factory.createScalar<int32_t>(handle);
        outputs[1] = factory.createScalar<int32_t>(error);
    }
private:
    void checkArguments(ArgumentList outputs, ArgumentList inputs) {
        std::shared_ptr<matlab::engine::MATLABEngine> matlabPtr = getEngine();
        ArrayFactory factory;
        if (inputs.size() != 3)
        {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("LJM_OpenS expects three input arguments") }));
        }
        if (inputs[0].getType() != ArrayType::MATLAB_STRING)
        {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("Input 0 (DeviceType) must be a string") }));
        }
        if (inputs[1].getType() != ArrayType::MATLAB_STRING)
        {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("Input 1 (ConnectionType) must be a string") }));
        }
        if (inputs[2].getType() != ArrayType::MATLAB_STRING)
        {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("Input 2 (Identifier) must be a string") }));
        }

        if (outputs.size() > 2) {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("Only two outputs (handle, error) are returned") }));
        }
    }
    // void ReturnError(LJM_ERROR_RETURN err) {
    // 	std::shared_ptr<matlab::engine::MATLABEngine> matlabPtr = getEngine();
    // 	ArrayFactory factory;
    // 	std::string message
    // 	matlabPtr->feval(u"error", 0,
    //             std::vector<Array>({ factory.createScalar("LJM Error %d: %s", err, message) }));
    // }
};