#include "mex.hpp"
#include "mexAdapter.hpp"
#include "LabJackM.h"

using namespace matlab::data;
using matlab::mex::ArgumentList;

class MexFunction : public matlab::mex::Function {
public:
    void operator()(ArgumentList outputs, ArgumentList inputs) {
        std::string registerName;
        int32_t handle, error;
        double value;
        ArrayFactory factory;

        checkArguments(outputs, inputs);
        TypedArray<int32_t> handleArr = inputs[0];
        handle = handleArr[0];
        TypedArray<MATLABString> nameArr = inputs[1];
        registerName = std::string(nameArr[0]);
        error = LJM_eReadName(handle, registerName.c_str(), &value);
        outputs[0] = factory.createScalar<double>(value);
        outputs[1] = factory.createScalar<int32_t>(error);
    }
private:
    void checkArguments(ArgumentList outputs, ArgumentList inputs) {
        std::shared_ptr<matlab::engine::MATLABEngine> matlabPtr = getEngine();
        ArrayFactory factory;
        if (inputs.size() != 2)
        {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("LJM_eReadName expects two input arguments") }));
        }
        if (inputs[0].getType() != ArrayType::INT32 ||
            inputs[0].getType() == ArrayType::COMPLEX_INT32)
        {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("Input 0 must be a noncomplex int32") }));
        }
        if (inputs[1].getType() != ArrayType::MATLAB_STRING)
        {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("Input 1 must be a string") }));
        }

        if (outputs.size() > 2) {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("Only two outputs are returned") }));
        }
    }
    // void ReturnError(LJM_ERROR_RETURN err) {
    //  std::shared_ptr<matlab::engine::MATLABEngine> matlabPtr = getEngine();
    //  ArrayFactory factory;
    //  std::string message
    //  matlabPtr->feval(u"error", 0,
    //             std::vector<Array>({ factory.createScalar("LJM Error %d: %s", err, message) }));
    // }
};