#include "mex.hpp"
#include "mexAdapter.hpp"
#include "LabJackM.h"

using namespace matlab::data;
using matlab::mex::ArgumentList;

class MexFunction : public matlab::mex::Function {
public:
    void operator()(ArgumentList outputs, ArgumentList inputs) {
        std::string registerName;
        int32_t handle, numFrames;
        double * aValues;
        LJM_ERROR_RETURN error;
        ArrayFactory factory;
        int errorAddress;
        std::shared_ptr<matlab::engine::MATLABEngine> matlabPtr = getEngine();

        checkArguments(outputs, inputs);
        TypedArray<int32_t> handleArr = inputs[0];
        handle = handleArr[0];
        TypedArray<int32_t> numFramesArr = inputs[1];
        numFrames = numFramesArr[0];
        char ** aNames =  (char **)malloc(numFrames * sizeof(char*));

        aValues = new double[numFrames];
        TypedArray<MATLABString> nameArr = inputs[2];
        TypedArray<double> valArr = inputs[3];
        for (int i = 0; i < numFrames; i++) {
            std::string temp = std::string(nameArr[i]);
            aNames[i] = (char *)malloc(temp.size());
            strcpy( aNames[i], temp.c_str());
            aValues[i] = double(valArr[i]);
        }
        error = LJM_eWriteNames(handle, numFrames, (const char **)aNames, aValues, &errorAddress);
        outputs[0] = factory.createScalar<int32_t>(errorAddress);
        outputs[1] = factory.createScalar<int32_t>(error);
        for (int i = 0; i < numFrames; i++) {
            free(aNames[i]);
        }
        free(aNames);
        delete[] aValues;
    }
private:
    void checkArguments(ArgumentList outputs, ArgumentList inputs) {
        std::shared_ptr<matlab::engine::MATLABEngine> matlabPtr = getEngine();
        ArrayFactory factory;
        if (inputs.size() != 4)
        {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("LJM_eWriteNames expects four input arguments") }));
        }
        if (inputs[0].getType() != ArrayType::INT32)
        {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("Input 0 (Handle) must be a noncomplex int32") }));
        }
        if (inputs[1].getType() != ArrayType::INT32)
        {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("Input 1 (NumFrames) must be a noncomplex int32") }));
        }
        if (inputs[2].getType() != ArrayType::MATLAB_STRING)
        {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("Input 2 must be an array of strings") }));
        }
        if (inputs[3].getType() != ArrayType::DOUBLE)
        {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("Input 3 must be an array of noncomplex double precision floating point numbers") }));
        }

        if (outputs.size() > 2) {
            matlabPtr->feval(u"error", 0,
                std::vector<Array>({ factory.createScalar("Only two outputs are returned") }));
        }
    }
    // void ReturnError(LJM_ERROR_RETURN err) {
    //  std::shared_ptr<matlab::engine::MATLABEngine> matlabPtr = getEngine();
    //  ArrayFactory factory;
    //  std::string message
    //  matlabPtr->feval(u"error", 0,
    //             std::vector<Array>({ factory.createScalar("LJM Error %d: %s", err, message) }));
    // }
};